@@ -52,6 +52,12 @@ try:
 except (ImportError,):
     pass
 
+def get_libvoikko_version():
+    if IMPORT_LIBVOIKKO_SUCCESSFUL:
+        return libvoikko.Voikko.getVersion()
+    else:
+        return '0'
+
 class HunspellSuggestTestCase(unittest.TestCase):
     def setUp(self):
         self.maxDiff = None
@@ -190,8 +196,8 @@ class HunspellSuggestTestCase(unittest.TestCase):
              ('kissamaiseksi',0)])
 
     @unittest.skipUnless(
-        IMPORT_LIBVOIKKO_SUCCESSFUL,
-        "Skipping because this test requires python3-libvoikko to work.")
+        get_libvoikko_version() >= '4.3',
+        "Skipping, requires python3-libvoikko version >= 4.3.")
     def test_fi_FI_voikko(self):
         d = hunspell_suggest.Dictionary('fi_FI')
         self.assertEqual(d.has_spellchecking(), True)
@@ -199,6 +205,7 @@ class HunspellSuggestTestCase(unittest.TestCase):
         self.assertEqual(
             h.suggest('kisssa'),
             [('kissa', -1),
+             ('Kiassa', -1),
              ('kissaa', -1),
              ('kisassa', -1),
              ('kisussa', -1)])
@@ -243,8 +250,8 @@ class HunspellSuggestTestCase(unittest.TestCase):
             ['camel', 'Camel'])
 
     @unittest.skipUnless(
-        IMPORT_LIBVOIKKO_SUCCESSFUL,
-        "Skipping because this test requires python3-libvoikko to work.")
+        get_libvoikko_version() >= '4.3',
+        "Skipping, requires python3-libvoikko version >= 4.3.")
     def test_fi_FI_spellcheck_voikko(self):
         d = hunspell_suggest.Dictionary('fi_FI')
         self.assertEqual(d.spellcheck_voikko('kissa'), True)
@@ -253,13 +260,13 @@ class HunspellSuggestTestCase(unittest.TestCase):
         self.assertEqual(d.spellcheck_voikko('Päivia'), False)
 
     @unittest.skipUnless(
-        IMPORT_LIBVOIKKO_SUCCESSFUL,
-        "Skipping because this test requires python3-libvoikko to work.")
+        get_libvoikko_version() >= '4.3',
+        "Skipping, requires python3-libvoikko version >= 4.3.")
     def test_fi_FI_spellcheck_suggest_voikko(self):
         d = hunspell_suggest.Dictionary('fi_FI')
         self.assertEqual(
             d.spellcheck_suggest_voikko('kisssa'),
-            ['kissa', 'kissaa', 'kisassa', 'kisussa'])
+            ['kissa', 'kissaa', 'kisassa', 'kisussa', 'Kiassa'])
 
     @unittest.skipUnless(
         itb_util.get_hunspell_dictionary_wordlist('sv_SE')[0],
