@@ -31,7 +31,7 @@ import unicodedata
 import unittest
 import subprocess
 import importlib
-import mock
+from unittest import mock
 
 from gi import require_version
 require_version('IBus', '1.0')
@@ -75,6 +75,12 @@ try:
 except (ImportError,):
     pass
 
+def get_libvoikko_version():
+    if IMPORT_LIBVOIKKO_SUCCESSFUL:
+        return libvoikko.Voikko.getVersion()
+    else:
+        return '0'
+
 @unittest.skipIf(Gdk.Display.open('') is None, 'Display cannot be opened.')
 class ItbTestCase(unittest.TestCase):
     '''
@@ -150,6 +156,8 @@ class ItbTestCase(unittest.TestCase):
             self.engine.get_tab_enable())
         self.orig_inline_completion = (
             self.engine.get_inline_completion())
+        self.orig_auto_capitalize = (
+            self.engine.get_auto_capitalize())
         self.orig_remember_last_used_preedit_ime = (
             self.engine.get_remember_last_used_preedit_ime())
         self.orig_page_size = (
@@ -189,6 +197,9 @@ class ItbTestCase(unittest.TestCase):
         self.engine.set_inline_completion(
             self.orig_inline_completion,
             update_gsettings=False)
+        self.engine.set_auto_capitalize(
+            self.orig_auto_capitalize,
+            update_gsettings=False)
         self.engine.set_remember_last_used_preedit_ime(
             self.orig_remember_last_used_preedit_ime,
             update_gsettings=False)
@@ -234,6 +245,8 @@ class ItbTestCase(unittest.TestCase):
             False, update_gsettings=False)
         self.engine.set_inline_completion(
             False, update_gsettings=False)
+        self.engine.set_auto_capitalize(
+            False, update_gsettings=False)
         self.engine.set_remember_last_used_preedit_ime(
             False, update_gsettings=False)
         self.engine.set_page_size(
@@ -830,6 +843,98 @@ class ItbTestCase(unittest.TestCase):
         self.assertEqual(self.engine.mock_committed_text, 'Barcelona ')
         self.assertEqual(self.engine.mock_preedit_text, '')
 
+    def test_auto_capitalize(self):
+        '''Test auto capitalization after punctuation
+        '''
+        self.engine.set_current_imes(
+            ['NoIME'], update_gsettings=False)
+        self.engine.set_dictionary_names(
+            ['en_US'], update_gsettings=False)
+        self.engine.set_auto_capitalize(True, update_gsettings=False)
+        self.engine.do_process_key_event(IBus.KEY_t, 0, 0)
+        self.engine.do_process_key_event(IBus.KEY_e, 0, 0)
+        self.engine.do_process_key_event(IBus.KEY_s, 0, 0)
+        self.engine.do_process_key_event(IBus.KEY_t, 0, 0)
+        self.assertEqual(self.engine.mock_preedit_text, 'test')
+        self.assertEqual(self.engine.mock_committed_text, '')
+        self.assertEqual(self.engine._candidates[0][0], 'test')
+        self.engine.do_process_key_event(IBus.KEY_space, 0, 0)
+        self.assertEqual(self.engine.mock_preedit_text, '')
+        self.assertEqual(self.engine.mock_committed_text, 'test ')
+        self.assertEqual(self.engine._candidates, [])
+        self.engine.do_process_key_event(IBus.KEY_t, 0, 0)
+        self.engine.do_process_key_event(IBus.KEY_e, 0, 0)
+        self.engine.do_process_key_event(IBus.KEY_s, 0, 0)
+        self.engine.do_process_key_event(IBus.KEY_t, 0, 0)
+        self.assertEqual(self.engine.mock_preedit_text, 'test')
+        self.assertEqual(self.engine.mock_committed_text, 'test ')
+        self.assertEqual(self.engine._candidates[0][0], 'test')
+        self.engine.do_process_key_event(IBus.KEY_period, 0, 0)
+        self.engine.do_process_key_event(IBus.KEY_space, 0, 0)
+        self.assertEqual(self.engine.mock_preedit_text, '')
+        self.assertEqual(self.engine.mock_committed_text, 'test test. ')
+        self.assertEqual(self.engine._candidates, [])
+        self.engine.do_process_key_event(IBus.KEY_t, 0, 0)
+        self.engine.do_process_key_event(IBus.KEY_e, 0, 0)
+        self.engine.do_process_key_event(IBus.KEY_s, 0, 0)
+        self.engine.do_process_key_event(IBus.KEY_t, 0, 0)
+        self.assertEqual(self.engine.mock_preedit_text, 'Test')
+        self.assertEqual(self.engine.mock_committed_text, 'test test. ')
+        self.assertEqual(self.engine._candidates[0][0], 'Test')
+        self.engine.do_process_key_event(IBus.KEY_space, 0, 0)
+        self.assertEqual(self.engine.mock_preedit_text, '')
+        self.assertEqual(self.engine.mock_committed_text, 'test test. Test ')
+        self.assertEqual(self.engine._candidates, [])
+        self.engine.do_process_key_event(IBus.KEY_t, 0, 0)
+        self.engine.do_process_key_event(IBus.KEY_e, 0, 0)
+        self.engine.do_process_key_event(IBus.KEY_s, 0, 0)
+        self.engine.do_process_key_event(IBus.KEY_t, 0, 0)
+        self.assertEqual(self.engine.mock_preedit_text, 'test')
+        self.assertEqual(self.engine.mock_committed_text, 'test test. Test ')
+        self.assertEqual(self.engine._candidates[0][0], 'test')
+        self.engine.do_process_key_event(IBus.KEY_space, 0, 0)
+        self.assertEqual(self.engine.mock_preedit_text, '')
+        self.assertEqual(self.engine.mock_committed_text,
+                         'test test. Test test ')
+        self.assertEqual(self.engine._candidates, [])
+        self.engine.do_process_key_event(IBus.KEY_period, 0, 0)
+        self.engine.do_process_key_event(IBus.KEY_space, 0, 0)
+        self.assertEqual(self.engine.mock_preedit_text, '')
+        self.assertEqual(self.engine.mock_committed_text,
+                         'test test. Test test . ')
+        self.assertEqual(self.engine._candidates, [])
+        self.engine.do_process_key_event(IBus.KEY_t, 0, 0)
+        self.engine.do_process_key_event(IBus.KEY_e, 0, 0)
+        self.engine.do_process_key_event(IBus.KEY_s, 0, 0)
+        self.engine.do_process_key_event(IBus.KEY_t, 0, 0)
+        self.assertEqual(self.engine.mock_preedit_text, 'Test')
+        self.assertEqual(self.engine.mock_committed_text,
+                         'test test. Test test . ')
+        self.assertEqual(self.engine._candidates[0][0], 'Test')
+        self.engine.do_process_key_event(IBus.KEY_space, 0, 0)
+        self.assertEqual(self.engine.mock_preedit_text, '')
+        self.assertEqual(self.engine.mock_committed_text,
+                         'test test. Test test . Test ')
+        self.engine.do_process_key_event(IBus.KEY_comma, 0, 0)
+        self.engine.do_process_key_event(IBus.KEY_space, 0, 0)
+        self.assertEqual(self.engine.mock_preedit_text, '')
+        self.assertEqual(self.engine.mock_committed_text,
+                         'test test. Test test . Test , ')
+        self.assertEqual(self.engine._candidates, [])
+        self.engine.do_process_key_event(IBus.KEY_h, 0, 0)
+        self.engine.do_process_key_event(IBus.KEY_e, 0, 0)
+        self.engine.do_process_key_event(IBus.KEY_l, 0, 0)
+        self.engine.do_process_key_event(IBus.KEY_l, 0, 0)
+        self.engine.do_process_key_event(IBus.KEY_o, 0, 0)
+        self.assertEqual(self.engine.mock_preedit_text, 'hello')
+        self.assertEqual(self.engine.mock_committed_text,
+                         'test test. Test test . Test , ')
+        self.assertEqual(self.engine._candidates[0][0], 'hello')
+        self.engine.do_process_key_event(IBus.KEY_space, 0, 0)
+        self.assertEqual(self.engine.mock_preedit_text, '')
+        self.assertEqual(self.engine.mock_committed_text,
+                         'test test. Test test . Test , hello ')
+
     def test_add_space_on_commit(self):
         '''Test new option to avoid adding spaces when committing by label
         (1-9 or F1-F9 key) or by mouse click.  See:
@@ -1358,8 +1463,8 @@ class ItbTestCase(unittest.TestCase):
         self.assertEqual(self.engine.mock_preedit_text, 'aṳ')
 
     @unittest.skipUnless(
-        IMPORT_LIBVOIKKO_SUCCESSFUL,
-        "Skipping because this test requires python3-libvoikko to work.")
+        get_libvoikko_version() >= '4.3',
+        "Skipping, requires python3-libvoikko version >= 4.3.")
     def test_voikko(self):
         self.engine.set_current_imes(
             ['NoIME'], update_gsettings=False)
@@ -1373,14 +1478,15 @@ class ItbTestCase(unittest.TestCase):
         self.engine.do_process_key_event(IBus.KEY_a, 0, 0)
         self.assertEqual(self.engine._candidates, [
             ('kissa', -1, '', False, True),
+            ('Kiassa', -1, '', False, True),
             ('kissaa', -1, '', False, True),
             ('kisassa', -1, '', False, True),
             ('kisussa', -1, '', False, True)
         ])
 
     @unittest.skipUnless(
-        IMPORT_LIBVOIKKO_SUCCESSFUL,
-        "Skipping because this test requires python3-libvoikko to work.")
+        get_libvoikko_version() >= '4.3',
+        "Skipping, requires python3-libvoikko version >= 4.3.")
     def test_voikko_en_GB_fi_FI(self):
         self.engine.set_current_imes(
             ['NoIME'], update_gsettings=False)
@@ -1395,6 +1501,7 @@ class ItbTestCase(unittest.TestCase):
         self.assertEqual(self.engine._candidates, [
             ('kiss', -1, '', False, True),
             ('kissa', -1, '', False, True),
+            ('Kiassa', -1, '', False, True),
             ('kissaa', -1, '', False, True),
             ('kisassa', -1, '', False, True),
             ('kisussa', -1, '', False, True)
